==> sclift-0.0.58.tar.gz: sclift-0.0.58/src/scLift/preprocessing/_scvi_.py <==
# -*- coding: UTF-8 -*-

import os
from typing import Optional

import numpy as np
import pandas as pd

from anndata import AnnData

from ..tool import umap, tsne
from ..util import path, project_name

from ykenan_file import StaticMethod
from ykenan_log import Logger

file_method = StaticMethod(f"{project_name}_preprocessing_scvi")
log = Logger(f"{project_name}_preprocessing_scvi")


def poisson_vi(
    adata: AnnData,
    max_epochs: int = 500,
    resolution: float = 0.6,
    latent_name: str = "latent",
    model_dir: Optional[path] = None
) -> AnnData:
    """
    PoissonVI processing of the data results in the current sample representation and peak difference data after Leiden clustering.
    :param adata: processing data;
    :param max_epochs: The maximum number of epochs for PoissonVI training;
    :param resolution: Resolution of the Leiden Cluster;
    :param latent_name: The name of latent representation;
    :param model_dir: The folder name saved by the training module;
    :return: Differential peak of clustering types.
    """
    log.info("Start PoissonVI")

    import scvi
    import scanpy as sc

    if resolution <= 0:
        log.error("The parameter `resolution` must be greater than zero.")
        raise ValueError("The parameter `resolution` must be greater than zero.")

    # PoissonVI, Binarization
    scvi.data.reads_to_fragments(adata)

    def __train__():
        # PoissonVI
        scvi.external.POISSONVI.setup_anndata(adata, layer="fragments")
        _model_ = scvi.external.POISSONVI(adata)
        _model_.train(max_epochs=max_epochs)
        return _model_

    if model_dir is not None:
        if os.path.exists(os.path.join(model_dir, "model.pt")):
            log.info(f"Due to the existence of file `model.pt`, it is loaded by default.")
            try:
                model = scvi.external.POISSONVI.load(model_dir, adata=adata)
            except FileExistsError as fee:
                log.error(f"File `model.pt` failed to load, you can execute `Poisson VI` again by deleting file `model.pt` ({model_dir}/model.pt).\n {fee}")
                raise ValueError(f"File `model.pt` failed to load, you can execute `Poisson VI` again by deleting file `model.pt` ({model_dir}/model.pt).")
        else:
            file_method.makedirs(model_dir)
            model = __train__()
            model.save(model_dir, overwrite=True)
    else:
        model = __train__()

    # latent space
    latent = model.get_latent_representation()
    adata.obsm[latent_name] = latent

    if "clusters" in adata.obs.columns:
        log.warn("Due to the original inclusion of the `clusters` column, the original `cluster` column name has been changed to `clusters_x`.")
        adata.obs["clusters_x"] = adata.obs["clusters"]

    # compute the k-nearest-neighbor graph that is used in both clustering and umap algorithms
    sc.pp.neighbors(adata, use_rep=latent_name)
    # cluster the space (we use a lower resolution to get fewer clusters than the default)
    sc.tl.leiden(adata, key_added="clusters", resolution=resolution)
    adata.obs["clusters"] = adata.obs["clusters"].astype(str)

    # umap
    try:
        data_umap = umap(adata.obsm[latent_name])
        adata.obsm["umap"] = data_umap
        adata.obs["latent_umap1"] = data_umap[:, 0]
        adata.obs["latent_umap2"] = data_umap[:, 1]
    except Exception as e:
        log.warn(f"UMAP error, your system does not support it, but it does not affect the process. Continue with execution: {e}")

    # tsne
    try:
        data_tsne = tsne(adata.obsm[latent_name])
        adata.obsm["tsne"] = data_tsne
        adata.obs["latent_tsne1"] = data_tsne[:, 0]
        adata.obs["latent_tsne2"] = data_tsne[:, 1]
    except Exception as e:
        log.warn(f"TSNE error, your system does not support it, but it does not affect the process. Continue with execution: {e}")

    clusters_list = list(set(adata.obs["clusters"]))
    clusters_list.sort()

    # differential peak
    da_peaks_all: dict = {}

    for cluster in clusters_list:
        log.info(f"Start difference peak: {cluster}/({', '.join(clusters_list)})")
        clusters_all = clusters_list.copy()
        clusters_all.remove(cluster)
        # differential peak
        da_peaks = model.differential_accessibility(adata, groupby="clusters", group1=cluster, two_sided=False)
        da_peaks_all.update({cluster: da_peaks[["bayes_factor", "emp_prob1", "group1"]]})

    adata.uns["da_peaks"] = da_peaks_all

    adata.uns["poisson_vi"] = {
        "model_dir": model_dir,
        "cluster_size": len(clusters_list),
        "latent_name": latent_name
    }
    peaks_info = adata.var.copy()
    peaks_info["index"] = peaks_info.index
    peaks_info.rename_axis("peak_index", inplace=True)

    matrix_bf = np.zeros((len(clusters_list), adata.shape[1]))
    matrix_ep1 = np.zeros((len(clusters_list), adata.shape[1]))

    for i in range(len(clusters_list)):
        cluster_info = da_peaks_all[clusters_list[i]]
        cluster_info["index"] = cluster_info.index
        cluster_info.rename_axis("cluster_index", inplace=True)
        cluster_info = pd.merge(left=peaks_info, right=cluster_info, left_on="index", right_on="index", how="left")
        matrix_bf[i, :] = cluster_info["bayes_factor"]
        matrix_ep1[i, :] = cluster_info["emp_prob1"]

    log.info("End PoissonVI")

    obs = pd.DataFrame(clusters_list, columns=["id"])
    obs.index = obs["id"].astype(str)
    da_peaks_adata = AnnData(matrix_bf, obs=obs, var=adata.var)
    da_peaks_adata.layers["emp_prob1"] = matrix_ep1
    da_peaks_adata.uns["latent_name"] = latent_name

    adata.uns["step"] = 1
    return da_peaks_adata

==> sclift-0.0.60.tar.gz: sclift-0.0.60/src/scLift/preprocessing/_scvi_.py <==
# -*- coding: UTF-8 -*-

import os
from typing import Optional

import numpy as np
import pandas as pd

from anndata import AnnData

from .. import util as ul
from ..tool import umap, tsne
from ..util import path

__name__: str = "preprocessing_scvi"


def poisson_vi(
    adata: AnnData,
    max_epochs: int = 500,
    resolution: float = 0.6,
    latent_name: str = "latent",
    model_dir: Optional[path] = None
) -> AnnData:
    """
    PoissonVI processing of the data results in the current sample representation and peak difference data after Leiden clustering.
    :param adata: processing data;
    :param max_epochs: The maximum number of epochs for PoissonVI training;
    :param resolution: Resolution of the Leiden Cluster;
    :param latent_name: The name of latent representation;
    :param model_dir: The folder name saved by the training module;
    :return: Differential peak of clustering types.
    """
    ul.log(__name__).info("Start PoissonVI")

    import scvi
    import scanpy as sc

    if resolution <= 0:
        ul.log(__name__).error("The parameter `resolution` must be greater than zero.")
        raise ValueError("The parameter `resolution` must be greater than zero.")

    # PoissonVI, Binarization
    scvi.data.reads_to_fragments(adata)

    def __train__():
        # PoissonVI
        scvi.external.POISSONVI.setup_anndata(adata, layer="fragments")
        _model_ = scvi.external.POISSONVI(adata)
        _model_.train(max_epochs=max_epochs)
        return _model_

    if model_dir is not None:
        if os.path.exists(os.path.join(model_dir, "model.pt")):
            ul.log(__name__).info(f"Due to the existence of file `model.pt`, it is loaded by default.")
            try:
                model = scvi.external.POISSONVI.load(model_dir, adata=adata)
            except FileExistsError as fee:
                ul.log(__name__).error(f"File `model.pt` failed to load, you can execute `Poisson VI` again by deleting file `model.pt` ({model_dir}/model.pt).\n {fee}")
                raise ValueError(f"File `model.pt` failed to load, you can execute `Poisson VI` again by deleting file `model.pt` ({model_dir}/model.pt).")
        else:
            ul.file_method(__name__).makedirs(model_dir)
            model = __train__()
            model.save(model_dir, overwrite=True)
    else:
        model = __train__()

    # latent space
    latent = model.get_latent_representation()
    adata.obsm[latent_name] = latent

    if "clusters" in adata.obs.columns:
        ul.log(__name__).warn("Due to the original inclusion of the `clusters` column, the original `cluster` column name has been changed to `clusters_x`.")
        adata.obs["clusters_x"] = adata.obs["clusters"]

    # compute the k-nearest-neighbor graph that is used in both clustering and umap algorithms
    sc.pp.neighbors(adata, use_rep=latent_name)
    # cluster the space (we use a lower resolution to get fewer clusters than the default)
    sc.tl.leiden(adata, key_added="clusters", resolution=resolution)
    adata.obs["clusters"] = adata.obs["clusters"].astype(str)

    # umap
    try:
        data_umap = umap(adata.obsm[latent_name])
        adata.obsm["umap"] = data_umap
        adata.obs["latent_umap1"] = data_umap[:, 0]
        adata.obs["latent_umap2"] = data_umap[:, 1]
    except Exception as e:
        ul.log(__name__).warn(f"UMAP error, your system does not support it, but it does not affect the process. Continue with execution: {e}")

    # tsne
    try:
        data_tsne = tsne(adata.obsm[latent_name])
        adata.obsm["tsne"] = data_tsne
        adata.obs["latent_tsne1"] = data_tsne[:, 0]
        adata.obs["latent_tsne2"] = data_tsne[:, 1]
    except Exception as e:
        ul.log(__name__).warn(f"TSNE error, your system does not support it, but it does not affect the process. Continue with execution: {e}")

    clusters_list = list(set(adata.obs["clusters"]))
    clusters_list.sort()

    # differential peak
    da_peaks_all: dict = {}

    for cluster in clusters_list:
        ul.log(__name__).info(f"Start difference peak: {cluster}/({', '.join(clusters_list)})")
        clusters_all = clusters_list.copy()
        clusters_all.remove(cluster)
        # differential peak
        da_peaks = model.differential_accessibility(adata, groupby="clusters", group1=cluster, two_sided=False)
        da_peaks_all.update({cluster: da_peaks[["bayes_factor", "emp_prob1", "group1"]]})

    adata.uns["da_peaks"] = da_peaks_all

    adata.uns["poisson_vi"] = {
        "model_dir": model_dir,
        "cluster_size": len(clusters_list),
        "latent_name": latent_name
    }
    peaks_info = adata.var.copy()
    peaks_info["index"] = peaks_info.index
    peaks_info.rename_axis("peak_index", inplace=True)

    matrix_bf = np.zeros((len(clusters_list), adata.shape[1]))
    matrix_ep1 = np.zeros((len(clusters_list), adata.shape[1]))

    for i in range(len(clusters_list)):
        cluster_info = da_peaks_all[clusters_list[i]]
        cluster_info["index"] = cluster_info.index
        cluster_info.rename_axis("cluster_index", inplace=True)
        cluster_info = pd.merge(left=peaks_info, right=cluster_info, left_on="index", right_on="index", how="left")
        matrix_bf[i, :] = cluster_info["bayes_factor"]
        matrix_ep1[i, :] = cluster_info["emp_prob1"]

    ul.log(__name__).info("End PoissonVI")

    obs = pd.DataFrame(clusters_list, columns=["id"])
    obs.index = obs["id"].astype(str)
    da_peaks_adata = AnnData(matrix_bf, obs=obs, var=adata.var)
    da_peaks_adata.layers["emp_prob1"] = matrix_ep1
    da_peaks_adata.uns["latent_name"] = latent_name

    adata.uns["step"] = 1
    return da_peaks_adata
